@@ -1,6 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989-1994, 2004-2013 Free Software Foundation, Inc.
+   Copyright (C) 1989-1994, 2004-2014, 2016-2017, 2020-2021 Free
+   Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -15,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "m4.h"
@@ -218,22 +219,17 @@ m4_tmpfile (int divnum)
     {
       output_temp_dir = create_temp_dir ("m4-", NULL, true);
       if (output_temp_dir == NULL)
-        M4ERROR ((EXIT_FAILURE, errno,
-                  "cannot create temporary file for diversion"));
+        m4_failure (errno, "cannot create temporary file for diversion");
       atexit (cleanup_tmpfile);
     }
   name = m4_tmpname (divnum);
   register_temp_file (output_temp_dir, name);
-  file = fopen_temp (name, O_BINARY ? "wb+" : "w+");
+  file = fopen_temp (name, O_BINARY ? "wb+e" : "w+e", false);
   if (file == NULL)
     {
       unregister_temp_file (output_temp_dir, name);
-      M4ERROR ((EXIT_FAILURE, errno,
-                "cannot create temporary file for diversion"));
+      m4_failure (errno, "cannot create temporary file for diversion");
     }
-  else if (set_cloexec_flag (fileno (file), true) != 0)
-    M4ERROR ((warning_status, errno,
-              "Warning: cannot protect diversion across forks"));
   return file;
 }
 
@@ -251,32 +247,26 @@ m4_tmpopen (int divnum, bool reread)
   if (tmp_file1_owner == divnum)
     {
       if (reread && fseeko (tmp_file1, 0, SEEK_SET) != 0)
-        m4_error (EXIT_FAILURE, errno,
-                  _("cannot seek within diversion"));
+        m4_failure (errno, _("cannot seek within diversion"));
       tmp_file2_recent = false;
       return tmp_file1;
     }
   else if (tmp_file2_owner == divnum)
     {
       if (reread && fseeko (tmp_file2, 0, SEEK_SET) != 0)
-        m4_error (EXIT_FAILURE, errno,
-                  _("cannot seek within diversion"));
+        m4_failure (errno, _("cannot seek within diversion"));
       tmp_file2_recent = true;
       return tmp_file2;
     }
   name = m4_tmpname (divnum);
   /* We need update mode, to avoid truncation.  */
-  file = fopen_temp (name, O_BINARY ? "rb+" : "r+");
+  file = fopen_temp (name, O_BINARY ? "rb+e" : "r+e", false);
   if (file == NULL)
-    M4ERROR ((EXIT_FAILURE, errno,
-              "cannot create temporary file for diversion"));
-  else if (set_cloexec_flag (fileno (file), true) != 0)
-    m4_error (0, errno, _("cannot protect diversion across forks"));
+    m4_failure (errno, "cannot create temporary file for diversion");
   /* Update mode starts at the beginning of the stream, but sometimes
      we want the end.  */
   else if (!reread && fseeko (file, 0, SEEK_END) != 0)
-    m4_error (EXIT_FAILURE, errno,
-              _("cannot seek within diversion"));
+    m4_failure (errno, _("cannot seek within diversion"));
   return file;
 }
 
@@ -348,8 +338,7 @@ m4_tmprename (int oldnum, int newnum)
       else
         {
           if (close_stream_temp (tmp_file1))
-            m4_error (EXIT_FAILURE, errno,
-                      _("cannot close temporary file for diversion"));
+            m4_failure (errno, _("cannot close temporary file for diversion"));
           tmp_file1_owner = 0;
         }
     }
@@ -361,16 +350,14 @@ m4_tmprename (int oldnum, int newnum)
       else
         {
           if (close_stream_temp (tmp_file2))
-            m4_error (EXIT_FAILURE, errno,
-                      _("cannot close temporary file for diversion"));
+            m4_failure (errno, _("cannot close temporary file for diversion"));
           tmp_file2_owner = 0;
         }
     }
   /* Either it is safe to rename an open file, or no one should have
      oldname open at this point.  */
   if (rename (oldname, newname))
-    m4_error (EXIT_FAILURE, errno,
-              _("cannot create temporary file for diversion"));
+    m4_failure (errno, _("cannot create temporary file for diversion"));
   unregister_temp_file (output_temp_dir, oldname);
   free (oldname);
   return m4_tmpopen (newnum, false);
@@ -483,8 +470,8 @@ make_room_for (int length)
           count = fwrite (selected_buffer, (size_t) selected_diversion->used,
                           1, selected_diversion->u.file);
           if (count != 1)
-            M4ERROR ((EXIT_FAILURE, errno,
-                      "ERROR: cannot flush diversion to temporary file"));
+            m4_failure (errno,
+                        "ERROR: cannot flush diversion to temporary file");
         }
 
       /* Reclaim the buffer space for other diversions.  */
@@ -578,7 +565,7 @@ output_text (const char *text, int length)
     {
       count = fwrite (text, length, 1, output_file);
       if (count != 1)
-        M4ERROR ((EXIT_FAILURE, errno, "ERROR: copying inserted file"));
+        m4_failure (errno, "ERROR: copying inserted file");
     }
   else
     {
@@ -630,14 +617,14 @@ shipout_text (struct obstack *obs, const char *text, int length, int line)
 
         /* In-line short texts.  */
 
-      case 8: OUTPUT_CHARACTER (*text); text++;
-      case 7: OUTPUT_CHARACTER (*text); text++;
-      case 6: OUTPUT_CHARACTER (*text); text++;
-      case 5: OUTPUT_CHARACTER (*text); text++;
-      case 4: OUTPUT_CHARACTER (*text); text++;
-      case 3: OUTPUT_CHARACTER (*text); text++;
-      case 2: OUTPUT_CHARACTER (*text); text++;
-      case 1: OUTPUT_CHARACTER (*text);
+      case 8: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 7: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 6: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 5: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 4: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 3: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 2: OUTPUT_CHARACTER (*text); text++; FALLTHROUGH;
+      case 1: OUTPUT_CHARACTER (*text); FALLTHROUGH;
       case 0:
         return;
 
@@ -826,7 +813,7 @@ insert_file (FILE *file)
     {
       length = fread (buffer, 1, sizeof buffer, file);
       if (ferror (file))
-        M4ERROR ((EXIT_FAILURE, errno, "error reading inserted file"));
+        m4_failure (errno, "error reading inserted file");
       if (length == 0)
         break;
       output_text (buffer, length);
@@ -993,11 +980,11 @@ freeze_diversions (FILE *file)
               struct stat file_stat;
               diversion->u.file = m4_tmpopen (diversion->divnum, true);
               if (fstat (fileno (diversion->u.file), &file_stat) < 0)
-                M4ERROR ((EXIT_FAILURE, errno, "cannot stat diversion"));
+                m4_failure (errno, "cannot stat diversion");
               if (file_stat.st_size < 0
                   || (file_stat.st_size + 0UL
                       != (unsigned long int) file_stat.st_size))
-                M4ERROR ((EXIT_FAILURE, 0, "diversion too large"));
+                m4_failure (0, "diversion too large");
               xfprintf (file, "D%d,%lu\n", diversion->divnum,
                         (unsigned long int) file_stat.st_size);
             }
